@@ -1,7 +1,7 @@
 /*
 htop - SignalsPanel.c
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -18,15 +18,14 @@ in the source distribution for its full text.
 #include "XUtils.h"
 
 
-Panel* SignalsPanel_new() {
+Panel* SignalsPanel_new(int preSelectedSignal) {
    Panel* this = Panel_new(1, 1, 1, 1, Class(ListItem), true, FunctionBar_newEnterEsc("Send   ", "Cancel "));
-   const int defaultSignal = SIGTERM;
    int defaultPosition = 15;
    unsigned int i;
    for (i = 0; i < Platform_numberOfSignals; i++) {
       Panel_set(this, i, (Object*) ListItem_new(Platform_signals[i].name, Platform_signals[i].number));
       // signal 15 is not always the 15th signal in the table
-      if (Platform_signals[i].number == defaultSignal) {
+      if (Platform_signals[i].number == preSelectedSignal) {
          defaultPosition = i;
       }
    }
