@@ -3,17 +3,22 @@
 /*
 htop - SignalsPanel.h
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include <signal.h>
+
 #include "Panel.h"
 
+
 typedef struct SignalItem_ {
    const char* name;
    int number;
 } SignalItem;
 
-Panel* SignalsPanel_new(void);
+#define SIGNALSPANEL_INITSELECTEDSIGNAL SIGTERM
+
+Panel* SignalsPanel_new(int preSelectedSignal);
 
 #endif
