@@ -1,7 +1,7 @@
 /*
 htop - TraceScreen.c
 (C) 2005-2006 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -10,6 +10,7 @@ in the source distribution for its full text.
 #include "TraceScreen.h"
 
 #include <assert.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <stdbool.h>
@@ -18,12 +19,10 @@ in the source distribution for its full text.
 #include <string.h>
 #include <unistd.h>
 #include <sys/select.h>
-#include <sys/time.h>
 #include <sys/wait.h>
 
 #include "CRT.h"
 #include "FunctionBar.h"
-#include "IncSet.h"
 #include "Panel.h"
 #include "ProvideCurses.h"
 #include "XUtils.h"
@@ -35,21 +34,23 @@ static const char* const TraceScreenKeys[] = {"F3", "F4", "F8", "F9", "Esc"};
 
 static const int TraceScreenEvents[] = {KEY_F(3), KEY_F(4), KEY_F(8), KEY_F(9), 27};
 
-TraceScreen* TraceScreen_new(Process* process) {
+TraceScreen* TraceScreen_new(const Process* process) {
    // This initializes all TraceScreen variables to "false" so only default = true ones need to be set below
    TraceScreen* this = xCalloc(1, sizeof(TraceScreen));
    Object_setClass(this, Class(TraceScreen));
    this->tracing = true;
    FunctionBar* fuBar = FunctionBar_new(TraceScreenFunctions, TraceScreenKeys, TraceScreenEvents);
    CRT_disableDelay();
-   return (TraceScreen*) InfoScreen_init(&this->super, process, fuBar, LINES - 2, "");
+   return (TraceScreen*) InfoScreen_init(&this->super, process, fuBar, LINES - 2, " ");
 }
 
 void TraceScreen_delete(Object* cast) {
    TraceScreen* this = (TraceScreen*) cast;
    if (this->child > 0) {
       kill(this->child, SIGTERM);
-      waitpid(this->child, NULL, 0);
+      while (waitpid(this->child, NULL, 0) == -1)
+         if (errno != EINTR)
+            break;
    }
 
    if (this->strace) {
@@ -87,16 +88,13 @@ bool TraceScreen_forkTracer(TraceScreen* this) {
       dup2(fdpair[1], STDERR_FILENO);
       close(fdpair[1]);
 
-      CRT_dropPrivileges();
-
       char buffer[32] = {0};
       xSnprintf(buffer, sizeof(buffer), "%d", this->super.process->pid);
       execlp("strace", "strace", "-T", "-tt", "-s", "512", "-p", buffer, NULL);
 
       // Should never reach here, unless execlp fails ...
       const char* message = "Could not execute 'strace'. Please make sure it is available in your $PATH.";
-      ssize_t written = write(STDERR_FILENO, message, strlen(message));
-      (void) written;
+      (void)! write(STDERR_FILENO, message, strlen(message));
 
       exit(127);
    }
