@@ -1,20 +1,19 @@
 /*
 htop - linux/Platform.c
 (C) 2014 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include "config.h"
 
-#include "Platform.h"
+#include "linux/Platform.h"
 
 #include <assert.h>
 #include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <inttypes.h>
-#include <limits.h>
 #include <math.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -31,41 +30,75 @@ in the source distribution for its full text.
 #include "DateTimeMeter.h"
 #include "DiskIOMeter.h"
 #include "HostnameMeter.h"
-#include "IOPriority.h"
-#include "IOPriorityPanel.h"
-#include "LinuxProcess.h"
-#include "LinuxProcessList.h"
+#include "HugePageMeter.h"
 #include "LoadAverageMeter.h"
 #include "Macros.h"
 #include "MainPanel.h"
 #include "Meter.h"
 #include "MemoryMeter.h"
+#include "MemorySwapMeter.h"
 #include "NetworkIOMeter.h"
 #include "Object.h"
 #include "Panel.h"
 #include "PressureStallMeter.h"
 #include "ProcessList.h"
 #include "ProvideCurses.h"
-#include "SELinuxMeter.h"
+#include "linux/SELinuxMeter.h"
 #include "Settings.h"
 #include "SwapMeter.h"
-#include "SystemdMeter.h"
+#include "SysArchMeter.h"
 #include "TasksMeter.h"
 #include "UptimeMeter.h"
 #include "XUtils.h"
-#include "ZramMeter.h"
-#include "ZramStats.h"
-
+#include "linux/IOPriority.h"
+#include "linux/IOPriorityPanel.h"
+#include "linux/LinuxProcess.h"
+#include "linux/LinuxProcessList.h"
+#include "linux/SystemdMeter.h"
+#include "linux/ZramMeter.h"
+#include "linux/ZramStats.h"
 #include "zfs/ZfsArcMeter.h"
 #include "zfs/ZfsArcStats.h"
 #include "zfs/ZfsCompressedArcMeter.h"
 
+#ifdef HAVE_LIBCAP
+#include <errno.h>
+#include <sys/capability.h>
+#endif
+
 #ifdef HAVE_SENSORS_SENSORS_H
 #include "LibSensors.h"
 #endif
 
+#ifndef O_PATH
+#define O_PATH         010000000 // declare for ancient glibc versions
+#endif
 
-const ProcessField Platform_defaultFields[] = { PID, USER, PRIORITY, NICE, M_VIRT, M_RESIDENT, M_SHARE, STATE, PERCENT_CPU, PERCENT_MEM, TIME, COMM, 0 };
+
+#ifdef HAVE_LIBCAP
+enum CapMode {
+   CAP_MODE_OFF,
+   CAP_MODE_BASIC,
+   CAP_MODE_STRICT
+};
+#endif
+
+bool Running_containerized = false;
+
+const ScreenDefaults Platform_defaultScreens[] = {
+   {
+      .name = "Main",
+      .columns = "PID USER PRIORITY NICE M_VIRT M_RESIDENT M_SHARE STATE PERCENT_CPU PERCENT_MEM TIME Command",
+      .sortKey = "PERCENT_CPU",
+   },
+   {
+      .name = "I/O",
+      .columns = "PID USER IO_PRIORITY IO_RATE IO_READ_RATE IO_WRITE_RATE PERCENT_SWAP_DELAY PERCENT_IO_DELAY Command",
+      .sortKey = "IO_RATE",
+   },
+};
+
+const unsigned int Platform_numberOfDefaultScreens = ARRAYSIZE(Platform_defaultScreens);
 
 const SignalItem Platform_signals[] = {
    { .name = " 0 Cancel",    .number = 0 },
@@ -111,36 +144,24 @@ static time_t Platform_Battery_cacheTime;
 static double Platform_Battery_cachePercent = NAN;
 static ACPresence Platform_Battery_cacheIsOnAC;
 
-void Platform_init(void) {
-   if (access(PROCDIR, R_OK) != 0) {
-      fprintf(stderr, "Error: could not read procfs (compiled to look in %s).\n", PROCDIR);
-      exit(1);
-   }
-
-#ifdef HAVE_SENSORS_SENSORS_H
-   LibSensors_init(NULL);
+#ifdef HAVE_LIBCAP
+static enum CapMode Platform_capabilitiesMode = CAP_MODE_BASIC;
 #endif
-}
-
-void Platform_done(void) {
-#ifdef HAVE_SENSORS_SENSORS_H
-   LibSensors_cleanup();
-#endif
-}
 
 static Htop_Reaction Platform_actionSetIOPriority(State* st) {
-   Panel* panel = st->panel;
+   if (Settings_isReadonly())
+      return HTOP_OK;
 
-   LinuxProcess* p = (LinuxProcess*) Panel_getSelected(panel);
+   const LinuxProcess* p = (const LinuxProcess*) Panel_getSelected((Panel*)st->mainPanel);
    if (!p)
       return HTOP_OK;
 
    IOPriority ioprio1 = p->ioPriority;
    Panel* ioprioPanel = IOPriorityPanel_new(ioprio1);
-   void* set = Action_pickFromVector(st, ioprioPanel, 21, true);
+   const void* set = Action_pickFromVector(st, ioprioPanel, 20, true);
    if (set) {
       IOPriority ioprio2 = IOPriorityPanel_getIOPriority(ioprioPanel);
-      bool ok = MainPanel_foreachProcess((MainPanel*)panel, LinuxProcess_setIOPriority, (Arg) { .i = ioprio2 }, NULL);
+      bool ok = MainPanel_foreachProcess(st->mainPanel, LinuxProcess_setIOPriority, (Arg) { .i = ioprio2 }, NULL);
       if (!ok) {
          beep();
       }
@@ -149,8 +170,40 @@ static Htop_Reaction Platform_actionSetIOPriority(State* st) {
    return HTOP_REFRESH | HTOP_REDRAW_BAR | HTOP_UPDATE_PANELHDR;
 }
 
+static bool Platform_changeAutogroupPriority(MainPanel* panel, int delta) {
+   if (LinuxProcess_isAutogroupEnabled() == false) {
+      beep();
+      return false;
+   }
+   bool anyTagged;
+   bool ok = MainPanel_foreachProcess(panel, LinuxProcess_changeAutogroupPriorityBy, (Arg) { .i = delta }, &anyTagged);
+   if (!ok)
+      beep();
+   return anyTagged;
+}
+
+static Htop_Reaction Platform_actionHigherAutogroupPriority(State* st) {
+   if (Settings_isReadonly())
+      return HTOP_OK;
+
+   bool changed = Platform_changeAutogroupPriority(st->mainPanel, -1);
+   return changed ? HTOP_REFRESH : HTOP_OK;
+}
+
+static Htop_Reaction Platform_actionLowerAutogroupPriority(State* st) {
+   if (Settings_isReadonly())
+      return HTOP_OK;
+
+   bool changed = Platform_changeAutogroupPriority(st->mainPanel, 1);
+   return changed ? HTOP_REFRESH : HTOP_OK;
+}
+
 void Platform_setBindings(Htop_Action* keys) {
    keys['i'] = Platform_actionSetIOPriority;
+   keys['{'] = Platform_actionLowerAutogroupPriority;
+   keys['}'] = Platform_actionHigherAutogroupPriority;
+   keys[KEY_F(19)] = Platform_actionLowerAutogroupPriority;  // Shift-F7
+   keys[KEY_F(20)] = Platform_actionHigherAutogroupPriority; // Shift-F8
 }
 
 const MeterClass* const Platform_meterTypes[] = {
@@ -162,6 +215,9 @@ const MeterClass* const Platform_meterTypes[] = {
    &LoadMeter_class,
    &MemoryMeter_class,
    &SwapMeter_class,
+   &MemorySwapMeter_class,
+   &SysArchMeter_class,
+   &HugePageMeter_class,
    &TasksMeter_class,
    &UptimeMeter_class,
    &BatteryMeter_class,
@@ -208,19 +264,25 @@ int Platform_getUptime() {
 }
 
 void Platform_getLoadAverage(double* one, double* five, double* fifteen) {
-   int activeProcs, totalProcs, lastProc;
-   *one = 0;
-   *five = 0;
-   *fifteen = 0;
-
    FILE* fd = fopen(PROCDIR "/loadavg", "r");
-   if (fd) {
-      int total = fscanf(fd, "%32lf %32lf %32lf %32d/%32d %32d", one, five, fifteen,
-         &activeProcs, &totalProcs, &lastProc);
-      (void) total;
-      assert(total == 6);
-      fclose(fd);
-   }
+   if (!fd)
+      goto err;
+
+   double scanOne, scanFive, scanFifteen;
+   int r = fscanf(fd, "%lf %lf %lf", &scanOne, &scanFive, &scanFifteen);
+   fclose(fd);
+   if (r != 3)
+      goto err;
+
+   *one = scanOne;
+   *five = scanFive;
+   *fifteen = scanFifteen;
+   return;
+
+err:
+   *one = NAN;
+   *five = NAN;
+   *fifteen = NAN;
 }
 
 int Platform_getMaxPid() {
@@ -235,12 +297,18 @@ int Platform_getMaxPid() {
    return maxPid;
 }
 
-double Platform_setCPUValues(Meter* this, int cpu) {
+double Platform_setCPUValues(Meter* this, unsigned int cpu) {
    const LinuxProcessList* pl = (const LinuxProcessList*) this->pl;
-   const CPUData* cpuData = &(pl->cpus[cpu]);
+   const CPUData* cpuData = &(pl->cpuData[cpu]);
    double total = (double) ( cpuData->totalPeriod == 0 ? 1 : cpuData->totalPeriod);
    double percent;
    double* v = this->values;
+
+   if (!cpuData->online) {
+      this->curItems = 0;
+      return NAN;
+   }
+
    v[CPU_METER_NICE] = cpuData->nicePeriod / total * 100.0;
    v[CPU_METER_NORMAL] = cpuData->userPeriod / total * 100.0;
    if (this->pl->settings->detailedCPUTime) {
@@ -282,18 +350,16 @@ void Platform_setMemoryValues(Meter* this) {
    const ProcessList* pl = this->pl;
    const LinuxProcessList* lpl = (const LinuxProcessList*) pl;
 
-   long int usedMem = pl->usedMem;
-   long int buffersMem = pl->buffersMem;
-   long int cachedMem = pl->cachedMem;
-   usedMem -= buffersMem + cachedMem;
-   this->total = pl->totalMem;
-   this->values[0] = usedMem;
-   this->values[1] = buffersMem;
-   this->values[2] = cachedMem;
+   this->total     = pl->totalMem;
+   this->values[0] = pl->usedMem;
+   this->values[1] = pl->buffersMem;
+   this->values[2] = pl->sharedMem;
+   this->values[3] = pl->cachedMem;
+   this->values[4] = pl->availableMem;
 
-   if (lpl->zfs.enabled != 0) {
+   if (lpl->zfs.enabled != 0 && !Running_containerized) {
       this->values[0] -= lpl->zfs.size;
-      this->values[2] += lpl->zfs.size;
+      this->values[3] += lpl->zfs.size;
    }
 }
 
@@ -301,6 +367,7 @@ void Platform_setSwapValues(Meter* this) {
    const ProcessList* pl = this->pl;
    this->total = pl->totalSwap;
    this->values[0] = pl->usedSwap;
+   this->values[1] = pl->cachedSwap;
 }
 
 void Platform_setZramValues(Meter* this) {
@@ -366,8 +433,8 @@ char* Platform_getProcessEnv(pid_t pid) {
  */
 char* Platform_getInodeFilename(pid_t pid, ino_t inode) {
    struct stat sb;
-   struct dirent *de;
-   DIR *dirp;
+   const struct dirent* de;
+   DIR* dirp;
    ssize_t len;
    int fd;
 
@@ -495,33 +562,27 @@ bool Platform_getDiskIO(DiskIOData* data) {
    if (!fd)
       return false;
 
-   unsigned long int read_sum = 0, write_sum = 0, timeSpend_sum = 0;
+   char lastTopDisk[32] = { '\0' };
+
+   unsigned long long int read_sum = 0, write_sum = 0, timeSpend_sum = 0;
    char lineBuffer[256];
    while (fgets(lineBuffer, sizeof(lineBuffer), fd)) {
       char diskname[32];
-      unsigned long int read_tmp, write_tmp, timeSpend_tmp;
-      if (sscanf(lineBuffer, "%*d %*d %31s %*u %*u %lu %*u %*u %*u %lu %*u %*u %lu", diskname, &read_tmp, &write_tmp, &timeSpend_tmp) == 4) {
+      unsigned long long int read_tmp, write_tmp, timeSpend_tmp;
+      if (sscanf(lineBuffer, "%*d %*d %31s %*u %*u %llu %*u %*u %*u %llu %*u %*u %llu", diskname, &read_tmp, &write_tmp, &timeSpend_tmp) == 4) {
          if (String_startsWith(diskname, "dm-"))
             continue;
 
-         /* only count root disks, e.g. do not count IO from sda and sda1 twice */
-         if ((diskname[0] == 's' || diskname[0] == 'h')
-             && diskname[1] == 'd'
-             && isalpha((unsigned char)diskname[2])
-             && isdigit((unsigned char)diskname[3]))
+         if (String_startsWith(diskname, "zram"))
             continue;
 
-         /* only count root disks, e.g. do not count IO from mmcblk0 and mmcblk0p1 twice */
-         if (diskname[0] == 'm'
-             && diskname[1] == 'm'
-             && diskname[2] == 'c'
-             && diskname[3] == 'b'
-             && diskname[4] == 'l'
-             && diskname[5] == 'k'
-             && isdigit((unsigned char)diskname[6])
-             && diskname[7] == 'p')
+         /* only count root disks, e.g. do not count IO from sda and sda1 twice */
+         if (lastTopDisk[0] && String_startsWith(diskname, lastTopDisk))
             continue;
 
+         /* This assumes disks are listed directly before any of their partitions */
+         String_safeStrncpy(lastTopDisk, diskname, sizeof(lastTopDisk));
+
          read_sum += read_tmp;
          write_sum += write_tmp;
          timeSpend_sum += timeSpend_tmp;
@@ -535,175 +596,119 @@ bool Platform_getDiskIO(DiskIOData* data) {
    return true;
 }
 
-bool Platform_getNetworkIO(unsigned long int* bytesReceived,
-                           unsigned long int* packetsReceived,
-                           unsigned long int* bytesTransmitted,
-                           unsigned long int* packetsTransmitted) {
+bool Platform_getNetworkIO(NetworkIOData* data) {
    FILE* fd = fopen(PROCDIR "/net/dev", "r");
    if (!fd)
       return false;
 
-   unsigned long int bytesReceivedSum = 0, packetsReceivedSum = 0, bytesTransmittedSum = 0, packetsTransmittedSum = 0;
+   memset(data, 0, sizeof(NetworkIOData));
    char lineBuffer[512];
    while (fgets(lineBuffer, sizeof(lineBuffer), fd)) {
       char interfaceName[32];
-      unsigned long int bytesReceivedParsed, packetsReceivedParsed, bytesTransmittedParsed, packetsTransmittedParsed;
-      if (sscanf(lineBuffer, "%31s %lu %lu %*u %*u %*u %*u %*u %*u %lu %lu",
+      unsigned long long int bytesReceived, packetsReceived, bytesTransmitted, packetsTransmitted;
+      if (sscanf(lineBuffer, "%31s %llu %llu %*u %*u %*u %*u %*u %*u %llu %llu",
                              interfaceName,
-                             &bytesReceivedParsed,
-                             &packetsReceivedParsed,
-                             &bytesTransmittedParsed,
-                             &packetsTransmittedParsed) != 5)
+                             &bytesReceived,
+                             &packetsReceived,
+                             &bytesTransmitted,
+                             &packetsTransmitted) != 5)
          continue;
 
       if (String_eq(interfaceName, "lo:"))
          continue;
 
-      bytesReceivedSum += bytesReceivedParsed;
-      packetsReceivedSum += packetsReceivedParsed;
-      bytesTransmittedSum += bytesTransmittedParsed;
-      packetsTransmittedSum += packetsTransmittedParsed;
+      data->bytesReceived += bytesReceived;
+      data->packetsReceived += packetsReceived;
+      data->bytesTransmitted += bytesTransmitted;
+      data->packetsTransmitted += packetsTransmitted;
    }
 
    fclose(fd);
 
-   *bytesReceived = bytesReceivedSum;
-   *packetsReceived = packetsReceivedSum;
-   *bytesTransmitted = bytesTransmittedSum;
-   *packetsTransmitted = packetsTransmittedSum;
    return true;
 }
 
 // Linux battery reading by Ian P. Hands (iphands@gmail.com, ihands@redhat.com).
 
-#define MAX_BATTERIES 64
 #define PROC_BATTERY_DIR PROCDIR "/acpi/battery"
 #define PROC_POWERSUPPLY_DIR PROCDIR "/acpi/ac_adapter"
+#define PROC_POWERSUPPLY_ACSTATE_FILE PROC_POWERSUPPLY_DIR "/AC/state"
 #define SYS_POWERSUPPLY_DIR "/sys/class/power_supply"
 
 // ----------------------------------------
 // READ FROM /proc
 // ----------------------------------------
 
-static unsigned long int parseBatInfo(const char* fileName, const unsigned short int lineNum, const unsigned short int wordNum) {
-   const char batteryPath[] = PROC_BATTERY_DIR;
-   DIR* batteryDir = opendir(batteryPath);
+static double Platform_Battery_getProcBatInfo(void) {
+   DIR* batteryDir = opendir(PROC_BATTERY_DIR);
    if (!batteryDir)
-      return 0;
+      return NAN;
 
-   char* batteries[MAX_BATTERIES];
-   unsigned int nBatteries = 0;
-   memset(batteries, 0, MAX_BATTERIES * sizeof(char*));
+   uint64_t totalFull = 0;
+   uint64_t totalRemain = 0;
 
-   while (nBatteries < MAX_BATTERIES) {
-      struct dirent* dirEntry = readdir(batteryDir);
-      if (!dirEntry)
-         break;
-
-      char* entryName = dirEntry->d_name;
+   struct dirent* dirEntry = NULL;
+   while ((dirEntry = readdir(batteryDir))) {
+      const char* entryName = dirEntry->d_name;
       if (!String_startsWith(entryName, "BAT"))
          continue;
 
-      batteries[nBatteries] = xStrdup(entryName);
-      nBatteries++;
-   }
-   closedir(batteryDir);
+      char filePath[256];
+      char bufInfo[1024] = {0};
+      xSnprintf(filePath, sizeof(filePath), "%s/%s/info", PROC_BATTERY_DIR, entryName);
+      ssize_t r = xReadfile(filePath, bufInfo, sizeof(bufInfo));
+      if (r < 0)
+         continue;
 
-   unsigned long int total = 0;
-   for (unsigned int i = 0; i < nBatteries; i++) {
-      char infoPath[30];
-      xSnprintf(infoPath, sizeof infoPath, "%s%s/%s", batteryPath, batteries[i], fileName);
+      char bufState[1024] = {0};
+      xSnprintf(filePath, sizeof(filePath), "%s/%s/state", PROC_BATTERY_DIR, entryName);
+      r = xReadfile(filePath, bufState, sizeof(bufState));
+      if (r < 0)
+         continue;
 
-      FILE* file = fopen(infoPath, "r");
-      if (!file)
-         break;
+      const char* line;
 
-      char* line = NULL;
-      for (unsigned short int j = 0; j < lineNum; j++) {
-         free(line);
-         line = String_readLine(file);
-         if (!line)
+      //Getting total charge for all batteries
+      char* buf = bufInfo;
+      while ((line = strsep(&buf, "\n")) != NULL) {
+         char field[100] = {0};
+         int val = 0;
+         if (2 != sscanf(line, "%99[^:]:%d", field, &val))
+            continue;
+
+         if (String_eq(field, "last full capacity")) {
+            totalFull += val;
             break;
+         }
       }
 
-      fclose(file);
+      //Getting remaining charge for all batteries
+      buf = bufState;
+      while ((line = strsep(&buf, "\n")) != NULL) {
+         char field[100] = {0};
+         int val = 0;
+         if (2 != sscanf(line, "%99[^:]:%d", field, &val))
+            continue;
 
-      if (!line)
-         break;
-
-      char* foundNumStr = String_getToken(line, wordNum);
-      const unsigned long int foundNum = atoi(foundNumStr);
-      free(foundNumStr);
-      free(line);
-
-      total += foundNum;
+         if (String_eq(field, "remaining capacity")) {
+            totalRemain += val;
+            break;
+         }
+      }
    }
 
-   for (unsigned int i = 0; i < nBatteries; i++)
-      free(batteries[i]);
+   closedir(batteryDir);
 
-   return total;
+   return totalFull > 0 ? ((double) totalRemain * 100.0) / (double) totalFull : NAN;
 }
 
 static ACPresence procAcpiCheck(void) {
-   ACPresence isOn = AC_ERROR;
-   const char* power_supplyPath = PROC_POWERSUPPLY_DIR;
-   DIR* dir = opendir(power_supplyPath);
-   if (!dir)
+   char buffer[1024] = {0};
+   ssize_t r = xReadfile(PROC_POWERSUPPLY_ACSTATE_FILE, buffer, sizeof(buffer));
+   if (r < 1)
       return AC_ERROR;
 
-   for (;;) {
-      struct dirent* dirEntry = readdir(dir);
-      if (!dirEntry)
-         break;
-
-      const char* entryName = dirEntry->d_name;
-
-      if (entryName[0] != 'A')
-         continue;
-
-      char statePath[256];
-      xSnprintf(statePath, sizeof(statePath), "%s/%s/state", power_supplyPath, entryName);
-      FILE* file = fopen(statePath, "r");
-      if (!file) {
-         isOn = AC_ERROR;
-         continue;
-      }
-      char* line = String_readLine(file);
-
-      fclose(file);
-
-      if (!line)
-         continue;
-
-      char* isOnline = String_getToken(line, 2);
-      free(line);
-
-      if (String_eq(isOnline, "on-line"))
-         isOn = AC_PRESENT;
-      else
-         isOn = AC_ABSENT;
-      free(isOnline);
-      if (isOn == AC_PRESENT)
-         break;
-   }
-
-   if (dir)
-      closedir(dir);
-
-   return isOn;
-}
-
-static double Platform_Battery_getProcBatInfo(void) {
-   const unsigned long int totalFull = parseBatInfo("info", 3, 4);
-   if (totalFull == 0)
-      return NAN;
-
-   const unsigned long int totalRemain = parseBatInfo("state", 5, 3);
-   if (totalRemain == 0)
-      return NAN;
-
-   return totalRemain * 100.0 / (double) totalFull;
+   return String_eq(buffer, "on-line") ? AC_PRESENT : AC_ABSENT;
 }
 
 static void Platform_Battery_getProcData(double* percent, ACPresence* isOnAC) {
@@ -716,7 +721,6 @@ static void Platform_Battery_getProcData(double* percent, ACPresence* isOnAC) {
 // ----------------------------------------
 
 static void Platform_Battery_getSysData(double* percent, ACPresence* isOnAC) {
-
    *percent = NAN;
    *isOnAC = AC_ERROR;
 
@@ -724,68 +728,81 @@ static void Platform_Battery_getSysData(double* percent, ACPresence* isOnAC) {
    if (!dir)
       return;
 
-   unsigned long int totalFull = 0;
-   unsigned long int totalRemain = 0;
-
-   for (;;) {
-      struct dirent* dirEntry = readdir(dir);
-      if (!dirEntry)
-         break;
+   uint64_t totalFull = 0;
+   uint64_t totalRemain = 0;
 
+   const struct dirent* dirEntry;
+   while ((dirEntry = readdir(dir))) {
       const char* entryName = dirEntry->d_name;
-      char filePath[256];
 
-      xSnprintf(filePath, sizeof filePath, SYS_POWERSUPPLY_DIR "/%s/type", entryName);
-
-      char type[8];
-      ssize_t r = xReadfile(filePath, type, sizeof(type));
-      if (r < 3)
+#ifdef HAVE_OPENAT
+      int entryFd = openat(dirfd(dir), entryName, O_DIRECTORY | O_PATH);
+      if (entryFd < 0)
          continue;
+#else
+      char entryFd[4096];
+      xSnprintf(entryFd, sizeof(entryFd), SYS_POWERSUPPLY_DIR "/%s", entryName);
+#endif
 
-      if (type[0] == 'B' && type[1] == 'a' && type[2] == 't') {
-         xSnprintf(filePath, sizeof filePath, SYS_POWERSUPPLY_DIR "/%s/uevent", entryName);
+      enum { AC, BAT } type;
+      if (String_startsWith(entryName, "BAT")) {
+         type = BAT;
+      } else if (String_startsWith(entryName, "AC")) {
+         type = AC;
+      } else {
+         char buffer[32];
+         ssize_t ret = xReadfileat(entryFd, "type", buffer, sizeof(buffer));
+         if (ret <= 0)
+            goto next;
 
+         /* drop optional trailing newlines */
+         for (char* buf = &buffer[(size_t)ret - 1]; *buf == '\n'; buf--)
+            *buf = '\0';
+
+         if (String_eq(buffer, "Battery"))
+            type = BAT;
+         else if (String_eq(buffer, "Mains"))
+            type = AC;
+         else
+            goto next;
+      }
+
+      if (type == BAT) {
          char buffer[1024];
-         r = xReadfile(filePath, buffer, sizeof(buffer));
-         if (r < 0) {
-            closedir(dir);
-            return;
-         }
+         ssize_t r = xReadfileat(entryFd, "uevent", buffer, sizeof(buffer));
+         if (r < 0)
+            goto next;
 
-         char* buf = buffer;
-         char* line = NULL;
          bool full = false;
          bool now = false;
-         int fullSize = 0;
+
+         double fullCharge = 0;
          double capacityLevel = NAN;
+         const char* line;
 
-         #define match(str,prefix) \
-            (String_startsWith(str,prefix) ? (str) + strlen(prefix) : NULL)
-
+         char* buf = buffer;
          while ((line = strsep(&buf, "\n")) != NULL) {
-            const char* ps = match(line, "POWER_SUPPLY_");
-            if (!ps)
+            char field[100] = {0};
+            int val = 0;
+            if (2 != sscanf(line, "POWER_SUPPLY_%99[^=]=%d", field, &val))
                continue;
-            const char* capacity = match(ps, "CAPACITY=");
-            if (capacity)
-               capacityLevel = atoi(capacity) / 100.0;
-            const char* energy = match(ps, "ENERGY_");
-            if (!energy)
-               energy = match(ps, "CHARGE_");
-            if (!energy)
+
+            if (String_eq(field, "CAPACITY")) {
+               capacityLevel = val / 100.0;
                continue;
-            const char* value = (!full) ? match(energy, "FULL=") : NULL;
-            if (value) {
-               fullSize = atoi(value);
-               totalFull += fullSize;
+            }
+
+            if (String_eq(field, "ENERGY_FULL") || String_eq(field, "CHARGE_FULL")) {
+               fullCharge = val;
+               totalFull += fullCharge;
                full = true;
                if (now)
                   break;
                continue;
             }
-            value = (!now) ? match(energy, "NOW=") : NULL;
-            if (value) {
-               totalRemain += atoi(value);
+
+            if (String_eq(field, "ENERGY_NOW") || String_eq(field, "CHARGE_NOW")) {
+               totalRemain += val;
                now = true;
                if (full)
                   break;
@@ -793,23 +810,18 @@ static void Platform_Battery_getSysData(double* percent, ACPresence* isOnAC) {
             }
          }
 
-         #undef match
-
          if (!now && full && !isnan(capacityLevel))
-            totalRemain += (capacityLevel * fullSize);
+            totalRemain += capacityLevel * fullCharge;
 
-      } else if (entryName[0] == 'A') {
+      } else if (type == AC) {
          if (*isOnAC != AC_ERROR)
-            continue;
-
-         xSnprintf(filePath, sizeof filePath, SYS_POWERSUPPLY_DIR "/%s/online", entryName);
+            goto next;
 
          char buffer[2];
-
-         r = xReadfile(filePath, buffer, sizeof(buffer));
+         ssize_t r = xReadfileat(entryFd, "online", buffer, sizeof(buffer));
          if (r < 1) {
-            closedir(dir);
-            return;
+            *isOnAC = AC_ERROR;
+            goto next;
          }
 
          if (buffer[0] == '0')
@@ -817,7 +829,11 @@ static void Platform_Battery_getSysData(double* percent, ACPresence* isOnAC) {
          else if (buffer[0] == '1')
             *isOnAC = AC_PRESENT;
       }
+
+next:
+      Compat_openatArgClose(entryFd);
    }
+
    closedir(dir);
 
    *percent = totalFull > 0 ? ((double) totalRemain * 100.0) / (double) totalFull : NAN;
@@ -852,3 +868,186 @@ void Platform_getBattery(double* percent, ACPresence* isOnAC) {
    Platform_Battery_cacheIsOnAC = *isOnAC;
    Platform_Battery_cacheTime = now;
 }
+
+void Platform_longOptionsUsage(const char* name)
+{
+#ifdef HAVE_LIBCAP
+   printf(
+"   --drop-capabilities[=off|basic|strict] Drop Linux capabilities when running as root\n"
+"                                off - do not drop any capabilities\n"
+"                                basic (default) - drop all capabilities not needed by %s\n"
+"                                strict - drop all capabilities except those needed for\n"
+"                                         core functionality\n", name);
+#else
+   (void) name;
+#endif
+}
+
+CommandLineStatus Platform_getLongOption(int opt, int argc, char** argv) {
+#ifndef HAVE_LIBCAP
+   (void) argc;
+   (void) argv;
+#endif
+
+   switch (opt) {
+#ifdef HAVE_LIBCAP
+      case 160: {
+         const char* mode = optarg;
+         if (!mode && optind < argc && argv[optind] != NULL &&
+            (argv[optind][0] != '\0' && argv[optind][0] != '-')) {
+            mode = argv[optind++];
+         }
+
+         if (!mode || String_eq(mode, "basic")) {
+            Platform_capabilitiesMode = CAP_MODE_BASIC;
+         } else if (String_eq(mode, "off")) {
+            Platform_capabilitiesMode = CAP_MODE_OFF;
+         } else if (String_eq(mode, "strict")) {
+            Platform_capabilitiesMode = CAP_MODE_STRICT;
+         } else {
+            fprintf(stderr, "Error: invalid capabilities mode \"%s\".\n", mode);
+            return STATUS_ERROR_EXIT;
+         }
+         return STATUS_OK;
+      }
+#endif
+
+      default:
+         break;
+   }
+   return STATUS_ERROR_EXIT;
+}
+
+#ifdef HAVE_LIBCAP
+static int dropCapabilities(enum CapMode mode) {
+
+   if (mode == CAP_MODE_OFF)
+      return 0;
+
+   /* capabilities we keep to operate */
+   const cap_value_t keepcapsStrict[] = {
+      CAP_DAC_READ_SEARCH,
+      CAP_SYS_PTRACE,
+   };
+   const cap_value_t keepcapsBasic[] = {
+      CAP_DAC_READ_SEARCH,   /* read non world-readable process files of other users, like /proc/[pid]/io */
+      CAP_KILL,              /* send signals to processes of other users */
+      CAP_SYS_NICE,          /* lower process nice value / change nice value for arbitrary processes */
+      CAP_SYS_PTRACE,        /* read /proc/[pid]/exe */
+#ifdef HAVE_DELAYACCT
+      CAP_NET_ADMIN,         /* communicate over netlink socket for delay accounting */
+#endif
+   };
+   const cap_value_t* const keepcaps = (mode == CAP_MODE_BASIC) ? keepcapsBasic : keepcapsStrict;
+   const size_t ncap = (mode == CAP_MODE_BASIC) ? ARRAYSIZE(keepcapsBasic) : ARRAYSIZE(keepcapsStrict);
+
+   cap_t caps = cap_init();
+   if (caps == NULL) {
+      fprintf(stderr, "Error: can not initialize capabilities: %s\n", strerror(errno));
+      return -1;
+   }
+
+   if (cap_clear(caps) < 0) {
+      fprintf(stderr, "Error: can not clear capabilities: %s\n", strerror(errno));
+      cap_free(caps);
+      return -1;
+   }
+
+   cap_t currCaps = cap_get_proc();
+   if (currCaps == NULL) {
+      fprintf(stderr, "Error: can not get current process capabilities: %s\n", strerror(errno));
+      cap_free(caps);
+      return -1;
+   }
+
+   for (size_t i = 0; i < ncap; i++) {
+      if (!CAP_IS_SUPPORTED(keepcaps[i]))
+         continue;
+
+      cap_flag_value_t current;
+      if (cap_get_flag(currCaps, keepcaps[i], CAP_PERMITTED, &current) < 0) {
+         fprintf(stderr, "Error: can not get current value of capability %d: %s\n", keepcaps[i], strerror(errno));
+         cap_free(currCaps);
+         cap_free(caps);
+         return -1;
+      }
+
+      if (current != CAP_SET)
+         continue;
+
+      if (cap_set_flag(caps, CAP_PERMITTED, 1, &keepcaps[i], CAP_SET) < 0) {
+         fprintf(stderr, "Error: can not set permitted capability %d: %s\n", keepcaps[i], strerror(errno));
+         cap_free(currCaps);
+         cap_free(caps);
+         return -1;
+      }
+
+      if (cap_set_flag(caps, CAP_EFFECTIVE, 1, &keepcaps[i], CAP_SET) < 0) {
+         fprintf(stderr, "Error: can not set effective capability %d: %s\n", keepcaps[i], strerror(errno));
+         cap_free(currCaps);
+         cap_free(caps);
+         return -1;
+      }
+   }
+
+   if (cap_set_proc(caps) < 0) {
+      fprintf(stderr, "Error: can not set process capabilities: %s\n", strerror(errno));
+      cap_free(currCaps);
+      cap_free(caps);
+      return -1;
+   }
+
+   cap_free(currCaps);
+   cap_free(caps);
+
+   return 0;
+}
+#endif
+
+bool Platform_init(void) {
+#ifdef HAVE_LIBCAP
+   if (dropCapabilities(Platform_capabilitiesMode) < 0)
+      return false;
+#endif
+
+   if (access(PROCDIR, R_OK) != 0) {
+      fprintf(stderr, "Error: could not read procfs (compiled to look in %s).\n", PROCDIR);
+      return false;
+   }
+
+#ifdef HAVE_SENSORS_SENSORS_H
+   LibSensors_init();
+#endif
+
+   char target[PATH_MAX];
+   ssize_t ret = readlink(PROCDIR "/self/ns/pid", target, sizeof(target) - 1);
+   if (ret > 0) {
+      target[ret] = '\0';
+
+      if (!String_eq("pid:[4026531836]", target)) { // magic constant PROC_PID_INIT_INO from include/linux/proc_ns.h#L46
+         Running_containerized = true;
+         return true; // early return
+      }
+   }
+
+   FILE* fd = fopen(PROCDIR "/1/mounts", "r");
+   if (fd) {
+      char lineBuffer[256];
+      while (fgets(lineBuffer, sizeof(lineBuffer), fd)) {
+         // detect lxc or overlayfs and guess that this means we are running containerized
+         if (String_startsWith(lineBuffer, "lxcfs ") || String_startsWith(lineBuffer, "overlay ")) {
+            Running_containerized = true;
+            break;
+         }
+      }
+      fclose(fd);
+   } // if (fd)
+
+   return true;
+}
+
+void Platform_done(void) {
+#ifdef HAVE_SENSORS_SENSORS_H
+   LibSensors_cleanup();
+#endif
+}
